@@ -6,7 +6,7 @@ std.fd <- function(fdobj)
   #  Return:
   #  STDFD ... a functional data for the standard deviation functions
 
-  #  Last modified 26 October 2005
+  #  Last modified 26 February 2007
 
   if (!(inherits(fdobj, "fd"))) stop(
 		"Argument  fdobj not a functional data object.")
@@ -15,32 +15,32 @@ std.fd <- function(fdobj)
   coefd    <- dim(coef)
   ndim     <- length(coefd)
   if (coefd[1] == 1) stop("Only one replication found.")
+  nrep     <- coefd[2]
+  ones     <- rep(1,nrep)
 
   basisobj <- fdobj$basis
   fdnames  <- fdobj$fdnames
   nbasis   <- basisobj$nbasis
   rangeval <- basisobj$rangeval
 
-  varbifd  <- var.fd(fdobj)
-
-  neval    <- 10*nbasis + 1
-  evalarg  <- seq(rangeval[1], rangeval[2], length=neval)
-  vararray <- eval.bifd(evalarg, evalarg, varbifd)
-  nvdim    <- length(dim(vararray))
+  neval   <- max(c(201,10*nbasis + 1))
+  evalarg <- seq(rangeval[1], rangeval[2], length=neval)
+  fdarray <- eval.fd(evalarg, fdobj)
 
   if (ndim == 2) {
-    stdmat  <- matrix(sqrt(diag(vararray)), neval, 1)
+    mnvec  <- (fdarray %*% ones)/nrep
+    resmat <- fdarray - (c(mnvec) %o% ones)
+    varvec <- (resmat^2 %*% ones)/(nrep-1)
+    stdmat <- sqrt(varvec)
   } else {
     nvar <- coefd[3]
     stdmat <- matrix(0, neval, nvar)
-    m <- 0
     for (j in 1:nvar) {
-      m <- m + j
-      if (nvdim == 3) {
-        stdmat[,j] <- sqrt(diag(varray[,,1,m]))
-      } else {
-        stdmat[,j] <- sqrt(diag(varray[,,m]))
-      }
+      mnvecj  <- (fdarray[,,j] %*% ones)/nrep
+      resmatj <- fdarray[,,j] - outer(c(mnvecj), ones)
+      varvecj <- (resmatj^2 %*% ones)/(nrep-1)
+      stdmatj <- sqrt(varvecj)
+      stdmat[,j] <- stdmatj
     }
   }
   stdcoef <- project.basis(stdmat, evalarg, basisobj)
