@@ -88,10 +88,6 @@ structure(c(0.5, 31.5, 59.5, 90.5, 120.5, 151.5, 181.5, 212.5,
 structure(c(15.5, 45, 74.5, 105, 135.5, 166, 196.5, 227.5, 258, 
 288.5, 319, 349.5), .Names = c("Jan", "Feb", "Mar", "Apr", "May", 
 "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"))
-`monthLetters` <-
-structure(c("j", "FALSE", "m", "A", "M", "J", "J", "A", "S", "O", 
-"N", "D"), .Names = c("Jan", "Feb", "Mar", "Apr", "May", "Jun", 
-"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"))
 `weeks` <-
 structure(c(0, 7.01923076923077, 14.0384615384615, 21.0576923076923, 
 28.0769230769231, 35.0961538461538, 42.1153846153846, 49.1346153846154, 
